# -*- coding: utf8 -*-

from django.core.management.base import NoArgsCommand

from os import listdir
from os.path import realpath, dirname, isdir, join as pathjoin
import re

from attachments.models import Attachment
from bbcode.bbcode import BBCode
from pages.models import PageWeb
from portailuser.models import Ecole


CHEMIN_RACINE = dirname(dirname(dirname(dirname(realpath(__file__)))))
CHEMIN_BASE = pathjoin(CHEMIN_RACINE, "archive_pages_web")
CHEMIN_PIECES_JOINTES = pathjoin(CHEMIN_BASE, "pieces_jointes")


class PieceJointe(object):
    def __init__(self, fichier, nom):
        self._fichier = fichier
        self.name = nom

    def chunks(self):
        while True:
            chunk = self._fichier.read(4096)
            if not chunk:
                break
            yield chunk


def decodage_pieces_jointes(match):
    nom_fichier = match.group(2)
    nom_unique = match.group(1)
    chemin_fichier = pathjoin(CHEMIN_PIECES_JOINTES, nom_unique)

    print(f"Lecture de la pièce jointe {nom_fichier} depuis le fichier {nom_unique}")

    with open(chemin_fichier, "r") as fichier:
        piece_jointe = Attachment.objects.upload_file("static_pages", PieceJointe(fichier, nom_fichier))
        return f"[attach]{piece_jointe.ident_main}[/attach]"


def parcourir_dossier(prefixe, pos=""):
    trouve = []

    for fichier in listdir(CHEMIN_BASE + "/" + prefixe + "/" + pos):
        chemin = CHEMIN_BASE + "/" + prefixe + "/" + pos + ("/" if pos else "") + fichier
        if isdir(chemin):
            trouve += parcourir_dossier(prefixe, pos + ("/" if pos else "") + fichier)
        elif fichier == "index.bbcode":
            trouve.append((chemin, pos + ("/" if pos else "")))
        elif fichier[-7:] == ".bbcode":
            trouve.append((chemin, pos + ("/" if pos else "") + fichier[:-7]))

    return trouve


class Command(NoArgsCommand):
    def handle_noargs(self, **kwargs):
        print(f"Lecture des pages dans {CHEMIN_BASE}…")

        for ecole in Ecole.objects.all():
            print(f"* {ecole.ident}…")

            if isdir(CHEMIN_BASE + "/" + ecole.ident):
                for chemin, adresse in parcourir_dossier(ecole.ident):
                    print(adresse)

                    with open(chemin, "rb") as fichier:
                        titre = fichier.readline().strip()
                        contenu = fichier.read()

                    contenu = re.sub(
                        r"\[attach\](\w+?.\w+?):(.*?)\[/attach\]",
                        decodage_pieces_jointes,
                        contenu,
                    )

                    bbcode = BBCode("web_page_attach", contenu, from_db=False)

                    try:
                        page = PageWeb.objects.get(ecole=ecole, adresse=adresse)
                    except PageWeb.DoesNotExist:
                        page = PageWeb(ecole=ecole, adresse=adresse)

                    page.titre = titre
                    page.contenu = bbcode
                    bbcode.validate_attachments("static_pages", page.pk)
                    page.save()
                    bbcode.commit_attachments("static_pages", page.pk)

        print("Importation terminée.")
